﻿Imports System.Windows
Imports System.ComponentModel
Imports System.Windows.Media.Imaging
Imports System.Collections.ObjectModel
Imports Telerik.Windows.Controls
Imports System.Collections.Generic
Imports System.Linq

Public Enum itemTypeEnum
  Group = 0
  Station = 1
  ControlCircuit = 2
  HeatingCircuit = 3
End Enum

Public Enum stateEnum
  Ok = 0
  [Error] = 1
End Enum

Public Class DataItem
  Implements INotifyPropertyChanged

  ' Icons für die unterschiedlichen Typen und Zustände
  Public okImage As New BitmapImage(New Uri("Images/ok.png", UriKind.Relative))
  Public ErrorImage As New BitmapImage(New Uri("Images/warning.png", UriKind.Relative))

  Public GroupImage As New BitmapImage(New Uri("Images/group.png", UriKind.Relative))
  Public StationImage As New BitmapImage(New Uri("Images/station.png", UriKind.Relative))
  Public ControlCircuitImage As New BitmapImage(New Uri("Images/controlcircuit.png", UriKind.Relative))
  Public HeatingCircuitImage As New BitmapImage(New Uri("Images/heatingcircuit.png", UriKind.Relative))

  Public Event PropertyChanged(sender As Object, e As System.ComponentModel.PropertyChangedEventArgs) Implements System.ComponentModel.INotifyPropertyChanged.PropertyChanged

  Private Sub NotifyPropertyChanged(info As [String])
    RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(info))
  End Sub

  Private _imgSize As Integer
  Public Property imgSize() As Integer
    Get
      Return _imgSize
    End Get
    Set(value As Integer)
      _imgSize = value
      NotifyPropertyChanged("imgSize")
    End Set
  End Property

  Public Sub New()
    imgSize = 22
    state = stateEnum.Ok
    ItemType = itemTypeEnum.Group
    Children = New ObservableCollection(Of DataItem)()
  End Sub

  Private _Id As Integer
  Public Property Id() As Integer
    Get
      Return _Id
    End Get
    Set(value As Integer)
      _Id = value
      NotifyPropertyChanged("Id")
    End Set
  End Property

  Private _ParentId As Integer
  Public Property ParentId() As Integer
    Get
      Return _ParentId
    End Get
    Set(value As Integer)
      _ParentId = value
      NotifyPropertyChanged("ParentId")
    End Set
  End Property

  Private _ItemType As itemTypeEnum
  Public Property ItemType() As itemTypeEnum
    Get
      Return _ItemType
    End Get
    Set(value As itemTypeEnum)
      _ItemType = value

      If _ItemType = itemTypeEnum.Group Then
        TypeImg = GroupImage
      End If
      If _ItemType = itemTypeEnum.Station Then
        TypeImg = StationImage
      End If
      If _ItemType = itemTypeEnum.ControlCircuit Then
        TypeImg = ControlCircuitImage
      End If
      If _ItemType = itemTypeEnum.HeatingCircuit Then
        TypeImg = HeatingCircuitImage
      End If
      NotifyPropertyChanged("TypeImg")
      NotifyPropertyChanged("ItemType")
    End Set
  End Property

  Private _state As stateEnum
  Public Property state() As stateEnum
    Get
      Return _state
    End Get
    Set(value As stateEnum)
      If value = stateEnum.Ok Then
        'If no childs, set state to Ok
        If Children Is Nothing Then
          _state = stateEnum.Ok
          NotifyPropertyChanged("state")
          Img = okImage

          'Set parent objects to Ok
          If Id > 0 Then
            Dim Parents As List(Of DataItem) = Owner.Where(Function(f) f.Id = ParentId).ToList()
            For Each item As DataItem In Parents
              item.state = stateEnum.Ok
              item.NotifyPropertyChanged("state")
            Next
          End If
        Else
          'If there are childs, check if all are ok
          If Children.Where(Function(f) (f.state = stateEnum.Ok)).Count() = Children.Count() Then
            'if so, set my state to ok
            _state = stateEnum.Ok
            NotifyPropertyChanged("state")
            Img = okImage

            '...and also the parent objects
            If Id > 0 Then
              Dim Parents As List(Of DataItem) = Owner.Where(Function(f) f.Id = ParentId).ToList()
              For Each item As DataItem In Parents
                item.state = stateEnum.Ok
                item.NotifyPropertyChanged("state")
              Next
            End If
          End If
        End If
      Else
        Img = ErrorImage
        _state = stateEnum.[Error]
        NotifyPropertyChanged("state")
        If Owner IsNot Nothing Then
          For Each item As DataItem In Owner.Where(Function(f) f.Id = Me.ParentId)
            item.state = stateEnum.[Error]
            item.NotifyPropertyChanged("state")
          Next
        End If
      End If
    End Set
  End Property

  Private _Text As String
  Public Property Text() As String
    Get
      Return _Text
    End Get
    Set(value As String)
      _Text = value
      NotifyPropertyChanged("Text")
    End Set
  End Property

  Private _Img As BitmapImage
  Public Property Img() As BitmapImage
    Get
      Return _Img
    End Get
    Set(value As BitmapImage)
      _Img = value
      NotifyPropertyChanged("Img")
    End Set
  End Property

  Private _TypeImg As BitmapImage
  Public Property TypeImg() As BitmapImage
    Get
      Return _TypeImg
    End Get
    Set(value As BitmapImage)
      _TypeImg = value
      NotifyPropertyChanged("TypeImg")
    End Set
  End Property

  Private _Owner As DataItemCollection
  Public Property Owner() As DataItemCollection
    Get
      Return _Owner
    End Get
    Protected Set(value As DataItemCollection)
      _Owner = value
      NotifyPropertyChanged("Owner")
    End Set
  End Property
  Friend Sub SetOwner(collection As DataItemCollection)
    Me.Owner = collection
  End Sub
  Friend Sub SetChildren(collection As ObservableCollection(Of DataItem))
    For Each item As DataItem In collection
      Me.Children.Add(item)
    Next
  End Sub

  Private m_children As ObservableCollection(Of DataItem)
  Public Property Children() As ObservableCollection(Of DataItem)
    Get
      Return Me.m_children
    End Get
    Set(value As ObservableCollection(Of DataItem))
      If Me.m_children IsNot value Then
        Me.m_children = value
        NotifyPropertyChanged("Children")
      End If
    End Set
  End Property

End Class
